const { Given, When, Then } = require("cypress-cucumber-preprocessor/steps");

// Variables to store data between steps
let newPostResponse = null;

Given('I have the endpoint {string}', (url) => {
  cy.wrap(url).as('apiEndpoint');
});

Given('I set the request headers as:', (table) => {
  const headers = table.hashes();
  cy.wrap(headers).as('requestHeaders');
});

Given('I set the request body as:', (body) => {
  cy.wrap(body).as('requestBody');
});

When('I make a POST request', () => {
  cy.get('@apiEndpoint').then((url) => {
    cy.get('@requestHeaders').then((headers) => {
      cy.get('@requestBody').then((body) => {
        cy.request({
          method: 'POST',
          url,
          headers,
          body: JSON.parse(body),
        }).as('response');
      });
    });
  });
});

Then('I should see a response with status code {int}', (statusCode) => {
  cy.get('@response').its('status').should('equal', statusCode);
});

Then('I save the response body as {string}', (varName) => {
  cy.get('@response').then((response) => {
    newPostResponse = response.body;
    cy.wrap(newPostResponse).as(varName);
  });
});

When('I make a GET request', () => {
  cy.get('@apiEndpoint').then((url) => {
    cy.request({
      method: 'GET',
      url,
    }).as('response');
  });
});

When('I make a PUT request', () => {
  cy.get('@apiEndpoint').then((url) => {
    cy.get('@requestHeaders').then((headers) => {
      cy.get('@requestBody').then((body) => {
        cy.request({
          method: 'PUT',
          url,
          headers,
          body: JSON.parse(body),
        }).as('response');
      });
    });
  });
});

When('I make a DELETE request', () => {
  cy.get('@apiEndpoint').then((url) => {
    cy.request({
      method: 'DELETE',
      url,
    }).as('response');
  });
});
